<?php
// $Id: views_tablesorter_plugin_style_tablesorter.inc,v 1.1 2010/05/31 13:48:00 rooby Exp $
/**
 * @file
 * Contains the tablesorter style plugin.
 */

/**
 * Style plugin to render each item as a row in a table
 * sortable using the tablesorter jQuery plugin.
 *
 * @ingroup views_style_plugins
 */
class views_tablesorter_plugin_style_tablesorter extends views_plugin_style {
  function option_definition() {
    $options = parent::option_definition();

    $options['columns'] = array('default' => array());
    $options['default_sort'] = array('default' => '');
    $options['info'] = array('default' => array());
    $options['override'] = array('default' => TRUE);
    $options['unsortable_cols'] = array('default' => array());
    $options['sticky'] = array('default' => FALSE);

    return $options;
  }

  /**
   * Normalize a list of columns based upon the fields that are
   * available. This compares the fields stored in the style handler
   * to the list of fields actually in the view, removing fields that
   * have been removed and adding new fields in their own column.
   *
   * - Each field must be in a column.
   * - Each column must be based upon a field, and that field
   *   is somewhere in the column.
   * - Any fields not currently represented must be added.
   * - Columns must be re-ordered to match the fields.
   *
   * @param $columns
   *   An array of all fields; the key is the id of the field and the
   *   value is the id of the column the field should be in.
   * @param $fields
   *   The fields to use for the columns. If not provided, they will
   *   be requested from the current display. The running render should
   *   send the fields through, as they may be different than what the
   *   display has listed due to access control or other changes.
   */
  function sanitize_columns($columns, $fields = NULL) {
    $sanitized = array();
    if ($fields === NULL) {
      $fields = $this->display->handler->get_option('fields');
    }

    // Preconfigure the sanitized array so that the order is retained.
    foreach ($fields as $field => $info) {
      // Set to itself so that if it isn't touched, it gets column
      // status automatically.
      $sanitized[$field] = $field;
    }

    foreach ($columns as $field => $column) {
      // first, make sure the field still exists.
      if (!isset($sanitized[$field])) {
        continue;
      }

      // If the field is the column, mark it so, or the column
      // it's set to is a column, that's ok
      if ($field == $column || $columns[$column] == $column && !empty($sanitized[$column])) {
        $sanitized[$field] = $column;
      }
      // Since we set the field to itself initially, ignoring
      // the condition is ok; the field will get its column
      // status back.
    }

    return $sanitized;
  }

  /**
   * Render the given style.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);
    $handlers = $this->display->handler->get_handlers('field');
    if (empty($handlers)) {
      $form['error_markup'] = array(
        '#value' => t('You need at least one field before you can configure your table settings'),
        '#prefix' => '<div class="error form-item description">',
        '#suffix' => '</div>',
      );
      return;
    }
    if (variable_get('views_no_javascript', FALSE)) {
      $form['error_markup'] = array(
        '#value' => t('The sorting and pager functionalities of this style will not currently work because the "Disable javascript with Views" option has been selected on the <a href="@tools-page">views tools page</a>.', array('@tools-page' => url('admin/build/views/tools'))),
        '#prefix' => '<div class="error form-item description">',
        '#suffix' => '</div>',
      );
    }

    $form['sticky'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enable Drupal style "sticky" table headers (Javascript)'),
      '#default_value' => !empty($this->options['sticky']),
      '#description' => t('(Sticky header effects will not be active for preview below, only on live output.)'),
    );

    $form['override'] = array(
      '#type' => 'checkbox',
      '#title' => t('Override normal sorting if click sorting is used'),
      '#default_value' => !empty($this->options['override']),
    );

    // This theme function is registered by views_tablesorter.module.
    $form['#theme'] = 'views_tablesorter_style_plugin_tablesorter';

    $columns = $this->sanitize_columns($this->options['columns']);

    // Create an array of allowed columns from the data we know:
    $field_names = $this->display->handler->get_field_labels();

    $priority_options = range(1, count($columns));
    array_unshift($priority_options, '<none>');

    foreach ($columns as $field => $column) {
      $safe = str_replace(array('][', '_', ' '), '-', $field);
      // The $id of the column for dependency checking.
      $id = 'edit-style-options-columns-' . $safe;

      $form['columns'][$field] = array(
        '#type' => 'select',
        '#options' => $field_names,
        '#default_value' => $column,
      );
      if ($handlers[$field]->click_sortable()) {
        $form['info'][$field]['sortable'] = array(
          '#type' => 'checkbox',
          '#default_value' => !empty($this->options['info'][$field]['sortable']),
          '#process' => array('views_process_dependency'),
          '#dependency' => array($id => array($field)),
        );
        $form['info'][$field]['default_priority'] = array(
          '#type' => 'select',
          '#options' => $priority_options,
          '#default_value' => isset($this->options['info'][$field]['default_priority']) ? $this->options['info'][$field]['default_priority'] : 0,
          '#process' => array('views_process_dependency'),
          '#dependency' => array($id => array($field)),
        );
        $form['info'][$field]['default_order'] = array(
          '#type' => 'select',
          '#options' => array(0 => t('Ascending'), 1 => t('Descending')),
          '#default_value' => isset($this->options['info'][$field]['default_order']) ? $this->options['info'][$field]['default_order'] : 0,
          '#process' => array('views_process_dependency'),
          '#dependency' => array($id => array($field)),
        );
      }
      $form['info'][$field]['separator'] = array(
        '#type' => 'textfield',
        '#size' => 10,
        '#default_value' => isset($this->options['info'][$field]['separator']) ? $this->options['info'][$field]['separator'] : '',
        '#process' => array('views_process_dependency'),
        '#dependency' => array($id => array($field)),
      );

      // Markup for the field name.
      $form['info'][$field]['name'] = array(
        '#value' => $field_names[$field],
      );
    }

    // This will be the default sort value.
    // It will be set in the form submit based on the individual field default values.
    $form['default_sort'] = array(
      '#type' => 'value',
      '#value' => isset($this->options['default_sort']) ? $this->options['default_sort'] : '',
    );
    $form['unsortable_cols'] = array(
      '#type' => 'value',
      '#value' => isset($this->options['unsortable_cols']) ? $this->options['unsortable_cols'] : '',
    );

    $form['description_markup'] = array(
      '#prefix' => '<div class="description form-item">',
      '#suffix' => '</div>',
      '#value' => t('Place fields into columns; you may combine multiple fields into the same column. If you do, the separator in the column specified will be used to separate the fields. Check the sortable boxes of the columns you want to make sortable, then you can set the default sort by selecting the priority of the sort fields and their sort order. You may control column order and field labels in the fields section.'),
    );
  }

  /**
   * Perform any necessary validation for the form.
   */
  function options_validate($form, &$form_state) {
    parent::options_validate($form, $form_state);
    // Make sure we have unique default priorities.
    $selected_priorities = array();
    $unique_error_set = FALSE;
    $unsortable_error_set = FALSE;
    foreach ($form_state['values']['style_options']['info'] as $field => $info) {
      if ($info['default_priority']) {
        // Make sure we don't have default sorts on unsortable columns.
        if (!isset($info['sortable']) || !$info['sortable']) {
          form_set_error("style_options][info][$field][default_priority", t("Unsortable columns cannot be used in the default sort."));
          if ($unsortable_error_set) {
            // So we don't get duplicate errors, unset any after the first.
            // It's a bit of a hack but hard to avoid with form_set_error().
            array_pop($_SESSION['messages']['error']);
          }
          $unsortable_error_set = TRUE;
        }
        // Make sure we have unique default priorities.
        else if (in_array($info['default_priority'], $selected_priorities)) {
          form_set_error("style_options][info][$field][default_priority", t("The same default priority cannot be used more than once."));
          if ($unique_error_set) {
            // So we don't get duplicate errors, unset any after the first.
            // It's a bit of a hack but hard to avoid with form_set_error().
            array_pop($_SESSION['messages']['error']);
          }
          $unique_error_set = TRUE;
        }
        $selected_priorities[] = $info['default_priority'];
      }
    }
  }

  /**
   * Perform any necessary changes to the form values prior to storage.
   */
  function options_submit($form, &$form_state) {
    parent::options_submit($form, $form_state);
    // Create the default sort list from the settings.
    $unsortable_cols = array();
    $default_sort = array();
    $defaults = array();
    $col_no = 0;

    foreach ($form_state['values']['style_options']['info'] as $info) {
      // Add to the unsortable cols list if not sortable.
      if (!isset($info['sortable']) || !$info['sortable']) {
        $unsortable_cols[$col_no] = array('sorter' => FALSE);
      }
      // Get the default sort options in a useable form.
      else if ($info['default_priority']) {
        $defaults[$info['default_priority']] = array($col_no, $info['default_order']);
      }
      $col_no++;
    }

    // Reorder defaults by priority.
    ksort($defaults);

    // Create the default sort list from the defaults.
    // If we aren't overriding the views sort options don't use our default sort.
    if ($form_state['values']['style_options']['override']) {
      foreach ($defaults as $default) {
        $default_sort[] = $default;
      }
    }

    // Store the settings in a form usable by the tablesorter jQuery plugin.
    $form_state['values']['style_options']['default_sort'] = serialize($default_sort);
    $form_state['values']['style_options']['unsortable_cols'] = serialize($unsortable_cols);
  }
}
